@@ -4,12 +4,14 @@ import time
 import serial
 from enum import Enum
 import logging
+import threading
+from threading import Thread
 
 from kuro.protocol import *
 
 class Gateway():   
 
-    def __init__(self, port, baudrate = 9600):
+    def __init__(self, port, baudrate = 9600), refresh_time = 10):
         """                
         Arguments:
             port {String} -- Serial port string as it is used in pyserial
@@ -20,26 +22,52 @@ class Gateway():
         """
         self.port = port
         self.baudrate = baudrate
-        self.configserial()
+        self.lock = threading.Lock()
+        self.status = 'off'
+        self.refresh_time = refresh_time
+        self.init_refresh()
+       
+
+        #self.configserial()
     
+    def init_refresh(self):
+        self.refresh = True
+        self.thread = Thread(target = self.refresh_power_status)
+        self.thread.start()
+        self.thread.join()
+    
+    def stop_refresh(self):
+        self.refresh = False
+
     def configserial(self):
         """
         Configure the serial port
         """
         try:
             if '://' in self.port:
-                self.ser = serial.serial_for_url(self.port)
+                self.ser = serial.serial_for_url(self.port, do_not_open = True)
+                self.ser.baudrate = self.baudrate
+                self.ser.parity = serial.PARITY_NONE
+                self.ser.stopbits=serial.STOPBITS_ONE
+                self.ser.bytesize=serial.EIGHTBITS
+                self.ser.timeout = 0
+                self.ser.xonxoff = False
+                self.ser.rtscts = False
+                self.ser.dsrdtr = False
+                self.ser.open()
+                logging.debug("Opened serial port")
             else:
-                self.ser = serial.Serial(self.port)
+                self.ser = serial.Serial(self.port, 
+                                self.baudrate, 
+                                bytesize = serial.EIGHTBITS, 
+                                timeout = 0, 
+                                parity = serial.PARITY_NONE,
+                                stopbits = serial.STOPBITS_ONE,
+                                xonxoff = False,
+                                rtscts = False,
+                                dsrdtr = False)
                 
-            self.ser.baudrate = self.baudrate
-            self.ser.parity = serial.PARITY_NONE
-            self.ser.stopbits=serial.STOPBITS_ONE
-            self.ser.bytesize=serial.EIGHTBITS
-            self.ser.timeout = 0
-            self.ser.xonxoff = False
-            self.ser.rtscts = False
-            self.ser.dsrdtr = False
+
 
         except Exception as e:            
             logging.error ('error open serial port: ' + str(e))
@@ -65,7 +93,6 @@ class Gateway():
         """
         commandstr = command.serialize()
         logging.info('Gateway writting: ' + str(commandstr))
-        print('Gateway writting: ' + str(commandstr))
 
         # try:
         #     self.configserial()
@@ -74,33 +101,27 @@ class Gateway():
         #     logging.error ('error open serial port: ' + str(e))
         #     exit()
         
-        if self.ser == None or not self.ser.isOpen():
+        if not hasattr(self, "ser") or self.ser == None or not self.ser.isOpen():
             self.configserial()
 
         try:
             self.ser.flushInput()
             self.ser.flushOutput()
-            
             self.ser.write(commandstr)
             time.sleep(0.3)
             response_str = "" 
             while True:
                 response = self.ser.readall()
                 response_str += str(response.decode())
-                #print(response_str)
-                logging.info('read data: ' + response_str)
                 if (response.decode()== ''):
                     break
                 
             #self.ser.close()
-            self.ser.flushOutput()
-            print(response_str)
+            #self.ser.flushOutput()
+            logging.debug(response_str)
             command.process_response(response_str)
-            #return process_response(response_str)
         except Exception as e1:
             logging.exception ("error communicating...: " + str(e1))
-        # else:
-        #     logging.error ("cannot open serial port")
         
         return None
     
@@ -114,12 +135,18 @@ class Gateway():
         self.executeCommand(command)
 
     def turn_on(self):
-        command = TurnOnCommand()
-        self.executeCommand(command)
-    
+        with self.lock:
+            self.configserial()
+            command = TurnOnCommand()
+            self.executeCommand(command)
+            self.ser.close()
+
     def turn_off(self):
-        command = TurnOffCommand()
-        self.executeCommand(command)
+        with self.lock:
+            self.configserial()
+            command = TurnOffCommand()
+            self.executeCommand(command)
+            self.ser.close()
 
     def volume_up(self):
         command = VolCommand("UP1")
@@ -164,17 +191,27 @@ class Gateway():
                 'maxVolume' : 60}
         
     def get_power_status(self):
-        command = PictureOffCommand()
-        self.executeCommand(command)
-        if command.response_type == ResponseType.SUCCESS:
-            return 'off'
-        elif command.response_type != ResponseType.ERROR:
-            return 'on'
+        with self.lock:
+            return self.status
+
+    def refresh_power_status(self):
+        while self.refresh:
+            with self.lock:
+                self.configserial()
+                command = PictureOffCommand()
+                self.executeCommand(command)
+                self.ser.close()
+                if command.response_type == ResponseType.SUCCESS:
+                    self.status = 'off'
+                elif command.response_type != ResponseType.ERROR:
+                    self.status = 'on'
+            time.sleep(self.refresh_time)
     
     def get_input_list(self):
         return [(member.describe(),member) for member in InputType]
 
     def get_status(self):
+        
         self.executeCommand(OsdCommand(OsdState.OFF))
         inputCommand = InputCommand()
         self.executeCommand(inputCommand)
@@ -194,11 +231,11 @@ if __name__ == '__main__':
 
     #manual = MethodCall("selve.GW.iveo.getIDs",[])
     gat = Gateway('rfc2217://192.168.1.20:7000')
-    power_status = gat.get_power_status()
-    print(power_status)
-    gat.turn_on()
-    vol_status = gat.get_volume_info()
-    print(vol_status)
+    # power_status = gat.get_power_status()
+    # print(power_status)
+    # gat.turn_on()
+    # vol_status = gat.get_volume_info()
+    # print(vol_status)
     #gat = Gateway('')
     #gat.executeCommand(MutedCommand(MutedState.ON))
 
